Attribute VB_Name = "Module1"
'Version date = 97.12.28
Option Explicit

Declare Sub FloodFill Lib "GDI32" (ByVal hDC As Long, ByVal X As Long, ByVal Y As Long, ByVal Color As Long)

Const ConNrSails As Integer = 13
Const conNrRudder As Integer = ConNrSails + 1
Const conNrTotalThrust As Integer = ConNrSails + 2
Const conNrTotalDrag As Integer = ConNrSails + 3
Const conNrNetThrust As Integer = ConNrSails + 4
Const conNrHeel As Integer = ConNrSails + 5
Const conNoOfIslands As Integer = 13
Const conPi As Single = 3.141592748
Const conDegToRad As Single = conPi / 180
Const conFact As Single = 0.25 'determines bluntness of stern on PlanView
Const conStep As Single = 0.5 'stepsize in feet for calculating effective sail area

Public HeadWidth(ConNrSails) As Single 'width of top edge of square sail
Public FootWidth(ConNrSails) As Single 'width of bottom edge of square sail
Public SailDepth(ConNrSails) As Single 'depth of sail (top<->bottom edge)
Public SailHead(ConNrSails) As Single 'height of top of sail above centre of rotation
Public SailFoot(ConNrSails) As Single 'height of bottom of sail above centre of rotation
Public MeanSailHeight(ConNrSails) As Single 'height om centre of (set part of) sail above centre of rotation
Public PlanHeadLeft(ConNrSails) As Single 'distance of top left from centre of rotation
Public PlanHeadRight(ConNrSails) As Single 'distance of top right from centre of rotation
Public PlanFootLeft(ConNrSails) As Single 'distance of bottom left from centre of rotation
Public PlanFootRight(ConNrSails) As Single 'distance of bottom right from centre of rotation
Public EffPlanHeadLeft(ConNrSails) As Single 'distance of top left of reefed sail from...
Public EffPlanHeadRight(ConNrSails) As Single 'distance of top right of reefed sail from...
Public WindHeadLeft(ConNrSails) As Single 'distance ... when seen from direction of wind
Public WindHeadRight(ConNrSails) As Single 'distance ... when seen from direction of wind
Public WindFootLeft(ConNrSails) As Single 'distance ... when seen from direction of wind
Public WindFootRight(ConNrSails) As Single 'distance ... when seen from direction of wind
Public SailStatus(ConNrSails) As Single 'set/furled/reefed etc.
Public SailAngle(ConNrSails) As Single 'set angle of sail: 0=square
Public PerpSailArea(ConNrSails + 1) As Single 'effective area of sail (area perp to wind)
Public ExposedFract(ConNrSails) As Single 'fraction of sail area delivering thrust
Public Sequence(ConNrSails, 3) As Integer 'order in which sails be drawn with wind from aft and forward
Public MastHeight(3) As Single 'calculated mast height (hull + sail depths + nominal gaps)

Public JibDepth(3) As Single 'vertical distance from foot to peak of jib
Public JibFootWidth(3) As Single 'horizontal distance from tack to clew of jib
Public JibTheorMaxSpeed(3) As Single 'theoretical max speed for this jib
Public JibSet As Integer 'the jib that is set: 1=large, 2=working, 3=storm
Public JibWidth As Single 'horizontal distance from peak to tack of set jib

Public Param$(9) 'name of these parameters
Public SailName$(ConNrSails) 'the names of the sails (not currently used)
Public EffSailHead(ConNrSails) As Single 'height of effective sail top above centre of rotation
Public TheorMaxSpeed(ConNrSails) As Single ' maximum windspeed for this sail
Public PerpFract(ConNrSails + 1) As Single ' fraction of wind perpendicular to sail
Public PerpForce(ConNrSails + 4) As Single ' wind force acting perpendicular to sail
Public HeadwayForce(ConNrSails + 4) As Single ' wind force acting to generate headway
Public LeewayForce(ConNrSails + 4) As Single ' wind force acting to generate leeway
Public RotatForce(ConNrSails + 5) As Single ' wind force acting to generate rotation
Public HeelForce(ConNrSails + 5) As Single ' wind force acting to generate heeling
Public MaxReefs(ConNrSails) As Single ' maximum number of reefs for this sail
Public MaxSpeed(ConNrSails) As Single 'actual (random adjusted) maximum speed the sail can withstand
Public SailWarning(ConNrSails + 5) As Single ' warning code for sails ashiver
Public LastValue$(ConNrSails + 5) 'last displayed value of current ssail parameter

Public IsleLat(conNoOfIslands) As Single, IsleLon(conNoOfIslands) As Single 'Island position
Public IsleRadius(conNoOfIslands) As Single 'Island position and radius
Public YardName$(0 To 3) 'name of the yard
Public YardAngle(0 To 3) As Integer 'brace angle of lowest yard on mast
Public YardLevel$(4) 'name of the yard level currently displayed
Public StatusName$(6) 'name of the sail status
Public SetFraction(6) As Single 'fraction of sail set by a particular SailStatus
Public MastDistance(4) As Single 'horizontal distance of mast (or metacentre of jib) from Centre of Lateral Resistance (+= forward)
Public LowestSailFoot(4) As Single 'height of foot of lowest sail on mast above see level
Public MaxYardAngle(0 To 3) As Integer 'maximum possible brace-up angle for lower yards
Public MaxJibAngle As Integer
Public MaxHelmAngle As Integer
Public Lat(2000) As Single, Lon(2000) As Single 'remember course
Public Xpos(12) As Single, Ypos(12) As Single ' coordinates of masts and yardarms
Public ShipLength As Single 'overall length of the ship
Public ShipWidth As Single
Public ShipDraught As Single
Public ShipTons As Single
Public QDeckLength As Single
Public FocleLength As Single
Public GunDeckHeight As Single 'height in feet above sealevel
Public QDeckHeight As Single 'height in feet above sealevel
Public RudderArea As Single
Public HelmAngle As Integer
Public Heading As Single 'in degrees
Public RequiredHeading As Single
Public HeadingDeviation As Single, LastDeviation As Single
Public Speed As Single 'in knots
Public LeewaySpeed As Single 'in knots
Public LeewayAngle As Single
Public RotationalSpeed As Single
Public HeelingAngle As Single
Public DegreesPerMin As Single
Public WindDirection  As Single 'in  (usual notation)
Public NextWindDirection As Single
Public NewWindDirection As Single
Public WindDirIncr As Single, DeltaWindDir As Single
Public WindSpeed As Single 'in knots
Public NextWindSpeed As Single, NewWindSpeed As Single
Public WindSpeedIncr As Single, DeltaWindSpeed As Single
Public DisplayParam As Integer 'the currently displayed sail parameter
Public YardLevelShown As Integer 'yards to display in PlanView (1=courses; 2=topsail; 3=topg'l'nt; 4=royal)
Public MadeSail As Integer 'sails sails (programmed setting 0...8)
Public MapWidth As Single 'width of map in nautical miles
Public BowFactor As Single 'modifies Frontal CSA (frontal aspect is not flat rectangle)
Public WaterDragFactor As Single 'arbitrary scale factor for drag of water (viscosity of water = 66 * viscosity of air)
Public RudderFactor As Single 'arbitrary scale factor for effectiveness of rudder vs sail (density of water = 770 * dens. air)
Public MassFactor As Single 'arbitrary conversion factor to convert tons to mass
Public RotatInertFactor As Single 'arbitrary scale factor for interia about vertical axis (turning)
Public RotatDragFactor As Single 'arbitrary scale factor for rotational drag (turning)
Public HeelAngleFactor As Single 'scale factor to generate appropriate heel
Public HeelInertFactor As Single 'arbitrary scale factor for inertia about longtitudinal axis (heel)
Public HeelDragFactor As Single 'arbitrary scale factor for drag against heeling
Public MyDir As String 'directory containing SURPRISE.CFG and BRIGHT.DIB
Public HelmInstruction$, YardInstruction$
Public SailXclick(10) As Single 'coordinate of click on sail plan
Public SailYclick(10) As Single 'coordinate of click on sail plan
Public SailClicks As Integer 'number of unprocessed clicks on sail plan
Public TentitiveHeading 'heading indicated by mouse pointer on plan view
Public CanShad(3, ConNrSails, 8) As Integer 'stores sails which can shadow other sails
Public DoShad(ConNrSails, 8) As Integer 'stores sails which actually shadow other sails
Public ShadStart(ConNrSails, 8) As Integer 'bottom of shaded band above sail foot
Public ShadEnd(ConNrSails, 8) As Integer 'top of shaded band above sail foot
Public ShadOfset(ConNrSails, 8) As Integer 'delta h between shadowed & shadowing sail

'derived parameters
Public CSABow As Single ' Cross Sectional Area
Public CSASide As Single
Public ShipMass As Single
Public RotationalInertia As Single
Public VerticalInertia As Single
Public HeelResistFactor As Single
Public MaxHeelMoment As Single

'screen positions of display items
Public SailParameter_x(ConNrSails + 5) As Single
Public SailParameter_y(ConNrSails + 5) As Single
Public First_y As Single, Heading_y As Single
Public Jib_x As Single, Jib_y As Single
Public Helm_y As Single
Public Yard_y(0 To 3) As Single
Public Forces_x As Single, Forces_y As Single, Left_x As Single, Wind_y As Single
Public Item_x As Single, Wind_x As Single, WindSpeed_x As Single
Public Selected_x As Single
Public Actual_x As Single, Colon_x As Single, Values_x As Single
Public Parameter_x As Single, Parameter_y As Single
Public Totals_x As Single, Totals_y As Single
Public AutoPilot_x As Single, AutoPilot_y As Single, Message_x As Single
Public ForceCol1 As Single, ForceCol2 As Single, ForceCol3 As Single
Public Tentitive_x As Single, Tentitive_y As Single
Public Tentitive_time As Integer

'to draw hull on PlanView
Public R1 As Single, R2 As Single, R3 As Single, R4 As Single
Public Le As Single, Offset As Single, Perp As Single
Public Radius As Single, ChordLength As Single

Public MapRadius As Single, TopEdge As Single, BottomEdge As Single
Public LeftEdge As Single, RightEdge As Single
Public AppWindAngle As Single, OldAppWindAngle As Single
Public AppWindDirection As Single
Public AppWindSpeed As Single
Public RelWindAngle As Single
Public MainMastHeight As Single
Public NrValuesToShow As Integer
Public ViewWidth As Single
Public ViewRadius As Single
Public WindArrowPoint As Single, WindArrowHead As Single
Public booPlanViewIsChanged As Boolean
Public booSternViewIsChanged As Boolean
Public booMapMustBePainted As Boolean
Public booHelmChange As Boolean
Public booYardChange(0 To 3) As Boolean
Public booDisplayForces As Boolean
Public booDisplayParams As Boolean
Public booParalax As Boolean 'true = sailplan shows view from wind direction

Public booShowSlipStream As Boolean
Public booStormJib As Boolean
Public booPaused As Boolean
Public booEstablished As Boolean
Public booMaxTrack As Boolean
Public booMapShift As Boolean
Public booHelmsman, booYardsmen As Boolean
Public booCloseHauled As Boolean
Public booWindSpeedChange As Boolean, booWindDirChange As Boolean
Public booNewValue As Boolean
Public TimeNow As Single
Public Increment As Single
Public EffectOfHeel As Single
Public NewHelmAngle As Single, NewYardAngle(0 To 3) As Integer
Public SelectedYard As Integer
Public ForwardThrust As Single
Public LeewardThrust As Single
Public RotationalThrust As Single
Public HeelingThrust As Single
Public ForwardDrag As Single
Public LeewardDrag As Single, RotationalDrag As Single
Public RightingMoment As Single, Resistance As Single
Public Counter As Integer 'number of map positions used
Public Longtitude As Single, Latitude As Single
Public OneThird As Single 'one third of current map width
Public COEForeAft As Single, COEHeight As Single, COELeeward As Single
Public WhichDialog As Integer
Public ThisConfig As Integer, ConfigItem As Integer, LastConfigItem As Integer
Public ItemValue$(60) 'used in frmConfigure
Public OldItemValue(60) As Single
Public TackStatus As Integer, MessageNr As Integer
Public OldJibStatus As Integer, OldJibAngle As Integer, OldSpankerStatus As Integer
Public ShipMessage$(6)
Public MessageTime(6) As Single
Public CycleStart As Single 'Time of start of current calculation cycle
'string versions of parameters used for display purposes
Public HeelEffect$, LastHeelEffect$
Public LastForwardThrust$, LastForwardDrag$, LastForwardTotal$
Public LastLeewardThrust$, LastLeewardDrag$, LastLeewardTotal$
Public LastRotationalThrust$, LastRotationalDrag$, LastRotationalTotal$
Public LastHeelingThrust$, LastResistance$, LastHeelingTotal$, LastRightingMoment$
Public OldHeading$, OldSpeed$
Public OldWindSpeed$, OldWindDirection$
Public OldLeewayAngle$, OldDegreesPerMin$, OldHeelingAngle$
Public OldWindDirText$, OldWindSpeedText$
'============================================================================

Public Sub LoadConfiguration()
'Version date = 97.12.28

Dim i As Integer
Dim Temp As String
Dim booConfigFile_l As Boolean

booConfigFile_l = True

On Error GoTo NoCfgFileOnLoad
Open MyDir & "surprise.cfg" For Input As #1
On Error GoTo 0

If booConfigFile_l = False Then Exit Sub
Input #1, Temp 'version identification
If Left(Temp, 8) <> "Surprise" Then
  Close #1
  MsgBox "Invalid Surprise.cfg file" & vbCrLf _
    & "(using default values)"
  Exit Sub
End If

Input #1, Temp: ShipLength = Val(Mid$(Temp, 22))
Input #1, Temp: QDeckLength = Val(Mid$(Temp, 22))
Input #1, Temp: FocleLength = Val(Mid$(Temp, 22))
Input #1, Temp: GunDeckHeight = Val(Mid$(Temp, 22))
Input #1, Temp: QDeckHeight = Val(Mid$(Temp, 22))
Input #1, Temp: ShipWidth = Val(Mid$(Temp, 22))
Input #1, Temp: ShipDraught = Val(Mid$(Temp, 22))
Input #1, Temp: ShipTons = Val(Mid$(Temp, 22))
Input #1, Temp: MastDistance(1) = Val(Mid$(Temp, 22))
Input #1, Temp: MastDistance(2) = Val(Mid$(Temp, 22))
Input #1, Temp: MastDistance(3) = Val(Mid$(Temp, 22))
Input #1, Temp: MastDistance(4) = Val(Mid$(Temp, 22))
Input #1, Temp: LowestSailFoot(1) = Val(Mid$(Temp, 22))
Input #1, Temp: LowestSailFoot(2) = Val(Mid$(Temp, 22))
Input #1, Temp: LowestSailFoot(3) = Val(Mid$(Temp, 22))
Input #1, Temp: LowestSailFoot(4) = Val(Mid$(Temp, 22))
Input #1, Temp: SetFraction(2) = Val(Mid$(Temp, 22))
Input #1, Temp: SetFraction(3) = Val(Mid$(Temp, 22))
Input #1, Temp: SetFraction(4) = Val(Mid$(Temp, 22))
Input #1, Temp: MaxYardAngle(0) = Val(Mid$(Temp, 22))
Input #1, Temp: MaxYardAngle(1) = Val(Mid$(Temp, 22))
Input #1, Temp: MaxYardAngle(2) = Val(Mid$(Temp, 22))
Input #1, Temp: MaxYardAngle(3) = Val(Mid$(Temp, 22))
Input #1, Temp: RudderArea = Val(Mid$(Temp, 22))
Input #1, Temp: MaxJibAngle = Val(Mid$(Temp, 22))
Input #1, Temp: MaxHelmAngle = Val(Mid$(Temp, 22))
For i = 1 To 6
  Input #1, Temp 'Skip a line
Next i
For i = 1 To ConNrSails
  Input #1, Temp
  HeadWidth(i) = Val(Mid$(Temp, 6, 9))
  FootWidth(i) = Val(Mid$(Temp, 16, 9))
  SailDepth(i) = Val(Mid$(Temp, 26, 9))
  MaxReefs(i) = Val(Mid$(Temp, 36, 9))
  SailStatus(i) = Val(Mid$(Temp, 46, 9))
  SailAngle(i) = Val(Mid$(Temp, 56, 9))
  TheorMaxSpeed(i) = Val(Mid$(Temp, 66, 9))
Next i
Input #1, Temp: JibSet = Val(Mid$(Temp, 22))
Input #1, Temp ' skip Line
For i = 1 To 3
  Input #1, Temp
  JibFootWidth(i) = Val(Mid$(Temp, 6, 9))
  JibDepth(i) = Val(Mid$(Temp, 16, 9))
  JibTheorMaxSpeed(i) = Val(Mid$(Temp, 26, 9))
Next i
Input #1, Temp: HelmAngle = Val(Mid$(Temp, 22))
Input #1, Temp: Heading = Val(Mid$(Temp, 22))
Input #1, Temp: Speed = Val(Mid$(Temp, 22))
Input #1, Temp: LeewaySpeed = Val(Mid$(Temp, 22))
Input #1, Temp: HeelingAngle = Val(Mid$(Temp, 22))
Input #1, Temp: MadeSail = Val(Mid$(Temp, 22))
Input #1, Temp: WindDirection = Val(Mid$(Temp, 22))
Input #1, Temp: WindSpeed = Val(Mid$(Temp, 22))
Input #1, Temp: DisplayParam = Val(Mid$(Temp, 22))
Input #1, Temp
If Mid$(Temp, 22) = "False" Then
  booShowSlipStream = False
Else
  booShowSlipStream = True
End If
Input #1, Temp: YardLevelShown = Val(Mid$(Temp, 22))
Input #1, Temp: MapWidth = Val(Mid$(Temp, 22))
Input #1, Temp: BowFactor = Val(Mid$(Temp, 22))
Input #1, Temp: WaterDragFactor = Val(Mid$(Temp, 22))
Input #1, Temp: RudderFactor = Val(Mid$(Temp, 22))
Input #1, Temp: MassFactor = Val(Mid$(Temp, 22))
Input #1, Temp: RotatInertFactor = Val(Mid$(Temp, 22))
Input #1, Temp: RotatDragFactor = Val(Mid$(Temp, 22))
Input #1, Temp: HeelAngleFactor = Val(Mid$(Temp, 22))
Input #1, Temp: HeelInertFactor = Val(Mid$(Temp, 22))
Input #1, Temp: HeelDragFactor = Val(Mid$(Temp, 22))
Input #1, Temp
For i = 1 To conNoOfIslands
  Input #1, Temp
  IsleLat(i) = Val(Mid$(Temp, 1, 9))
  IsleLon(i) = Val(Mid$(Temp, 12, 9))
  IsleRadius(i) = Val(Mid$(Temp, 24, 9))
Next i
Close #1
Exit Sub

NoMoreIslands:
On Error GoTo 0
Close #1
Exit Sub

NoCfgFileOnLoad:
On Error GoTo 0
beep: MsgBox "No Surprise.cfg. Using default values", , "Configuration source"
End Sub

Public Sub SaveConfiguration()
'Version date = 98.01.02

Dim i As Integer

On Error GoTo NoFile
Kill MyDir & "surprise.cfg"

NoFile:
On Error GoTo 0

Open MyDir & "surprise.cfg" For Output As 1
Print #1, "Surprise 19980102"
Print #1, "Ship length         : ";
Print #1, Format(ShipLength, "0000;-000")
Print #1, "Quarterdeck length  : ";
Print #1, Format(QDeckLength, "0000;-000")
Print #1, "Forecastle length   : ";
Print #1, Format(FocleLength, "0000;-000")
Print #1, "Gundeck height      : ";
Print #1, Format(GunDeckHeight, "0000;-000")
Print #1, "Quaterdeck height   : ";
Print #1, Format(QDeckHeight, "0000;-000")
Print #1, "Ship width          : ";
Print #1, Format(ShipWidth, "0000;-000")
Print #1, "Ship draught        : ";
Print #1, Format(ShipDraught, "0000;-000")
Print #1, "Ship tonnage        : ";
Print #1, Format(ShipTons, "0000;-000")
Print #1, "Fore mast distance  : ";
Print #1, Format(MastDistance(1), "0000;-000")
Print #1, "Main mast distance  : ";
Print #1, Format(MastDistance(2), "0000;-000")
Print #1, "Mizzen mast distance: ";
Print #1, Format(MastDistance(3), "0000;-000")
Print #1, "Jib clew distance   : ";
Print #1, Format(MastDistance(4), "0000;-000")
Print #1, "Forecourse foot hgt : ";
Print #1, Format(LowestSailFoot(1), "0000;-000")
Print #1, "Maincourse foot hgt : ";
Print #1, Format(LowestSailFoot(2), "0000;-000")
Print #1, "Mizzencourse foot ht: ";
Print #1, Format(LowestSailFoot(3), "0000;-000")
Print #1, "Jib foot height     : ";
Print #1, Format(LowestSailFoot(4), "0000;-000")
Print #1, "Set fract 1 reef    : ";
Print #1, Prnt(Format(SetFraction(2), "0.00;-000"))
Print #1, "Set fract 2 reefs   : ";
Print #1, Prnt(Format(SetFraction(3), "0.00;-000"))
Print #1, "Set fract 3 reefs   : ";
Print #1, Prnt(Format(SetFraction(4), "0.00;-000"))
Print #1, "Max foreyard angle  : ";
Print #1, Format(MaxYardAngle(0), "0000;-000")
Print #1, "Max mainyard angle  : ";
Print #1, Format(MaxYardAngle(1), "0000;-000")
Print #1, "Max crossjack angle : ";
Print #1, Format(MaxYardAngle(2), "0000;-000")
Print #1, "Max mizzenboom angle: ";
Print #1, Format(MaxYardAngle(3), "0000;-000")
Print #1, "Rudder area         : ";
Print #1, Format(RudderArea, "0000;-000")
Print #1, "Max jib angle       : ";
Print #1, Format(MaxJibAngle, "0000;-000")
Print #1, "Max helm angle      : ";
Print #1, Format(MaxHelmAngle, "0000;-000")
For i = 1 To 5
  Print #1, "Reserved for expansion"
Next i
Print #1, "Sail HeadWidth FootWidth     Depth  MaxReefs    Status YardAngle Max Speed"
For i = 1 To ConNrSails
  Print #1, Format(i, "  00");
  Print #1, Format(HeadWidth(i), "     00000");
  Print #1, Format(FootWidth(i), "     00000");
  Print #1, Format(SailDepth(i), "     00000");
  Print #1, Format(MaxReefs(i), "     00000");
  Print #1, Format(SailStatus(i), "     00000");
  Print #1, Format(SailAngle(i), "     00000;     -0000");
  Print #1, Format(TheorMaxSpeed(i), "     00000")
Next i
Print #1, "Jib set             : ";
Print #1, Format(JibSet, "0000;-000")
Print #1, "Jib  Footwidth     Depth Max Speed"
For i = 1 To 3
  Print #1, Format(i, "  00");
  Print #1, Format(JibFootWidth(i), "     00000");
  Print #1, Format(JibDepth(i), "     00000");
  Print #1, Format(JibTheorMaxSpeed(i), "     00000")
Next i
Print #1, "Helm angle          : ";
Print #1, Format(HelmAngle, "0000;-000")
Print #1, "Heading             : ";
Print #1, Prnt(Format(Heading, "0000.00;-000.00"))
Print #1, "Speed               : ";
Print #1, Prnt(Format(Speed, "0000.00;-000.00"))
Print #1, "Leeway speed        : ";
Print #1, Prnt(Format(LeewaySpeed, "0000.00;-000.00"))
Print #1, "Heel angle          : ";
Print #1, Prnt(Format(HeelingAngle, "0000.00;-000.00"))
Print #1, "Sail pattern set    : ";
Print #1, Format(MadeSail, "0000;-000")
Print #1, "Wind direction      : ";
Print #1, Format(WindDirection, "0000;-000")
Print #1, "Wind speed          : ";
Print #1, Format(WindSpeed, "0000;-000")
Print #1, "Parameter to display: ";
Print #1, Format(DisplayParam, "0000;-000")
Print #1, "Show slipstream?    : ";
Print #1, booShowSlipStream
Print #1, "Yards to display    : ";
Print #1, Format(YardLevelShown, "0000;-000")
Print #1, "Map width           : ";
Print #1, Prnt(Format(MapWidth, "00.000;-0.000"))
Print #1, "Bow factor          : ";
Print #1, Prnt(Format(BowFactor, "000000.00000;-00000.00000"))
Print #1, "Water drag factor   : ";
Print #1, Prnt(Format(WaterDragFactor, "000000.00000;-00000.00000"))
Print #1, "Rudder factor       : ";
Print #1, Prnt(Format(RudderFactor, "000000.00000;-00000.00000"))
Print #1, "Mass factor         : ";
Print #1, Prnt(Format(MassFactor, "000000.00000;-00000.00000"))
Print #1, "Rot. inertia factor : ";
Print #1, Prnt(Format(RotatInertFactor, "000000.00000;-00000.00000"))
Print #1, "Rot. drag factor    : ";
Print #1, Prnt(Format(RotatDragFactor, "000000.00000;-00000.00000"))
Print #1, "Heel angle factor   : ";
Print #1, Prnt(Format(HeelAngleFactor, "000000.00000;-00000.00000"))
Print #1, "Heel inertia factor : ";
Print #1, Prnt(Format(HeelInertFactor, "000000.00000;-00000.00000"))
Print #1, "Heel drag factor    : ";
Print #1, Prnt(Format(HeelDragFactor, "000000.00000;-00000.00000"))
Print #1, "Isle Lat   Isle Lon    Isle Radius"
For i = 1 To conNoOfIslands
  Print #1, Prnt(Format(IsleLat(i), "000.0000;-00.0000;000.0000"));
  Print #1, Prnt(Format(IsleLon(i), "   000.0000;   -00.0000;   000.0000"));
  Print #1, Prnt(Format(IsleRadius(i), "    000.0000;    -00.0000;    000.0000"))
Next i
Close 1

End Sub

Public Sub DefaultParameters()
'Version date = 97.12.28

'Replace this statement with your path
MyDir = ""

Dim i As Integer

SailName$(1) = "Fore Royal"
SailName$(2) = "Fore Topgallant"
SailName$(3) = "Fore Topsail"
SailName$(4) = "Foresail"
SailName$(5) = "Main Royal"
SailName$(6) = "Main Topgallant"
SailName$(7) = "Main Topsail"
SailName$(8) = "Mainsail"
SailName$(9) = "Mizzen Royal"
SailName$(10) = "Mizzen Topgallant"
SailName$(11) = "Mizzen Topsail"
SailName$(12) = "Mizzen sail"
SailName$(13) = "Working jib"

Param$(1) = "Yard Angle": '0=square; +=starboard braced up
Param$(2) = "Exposed percentage"
Param$(3) = "Effective area"
Param$(4) = "Useful percentage of force"
Param$(5) = "Useful force"
Param$(6) = "Forward thrust"
Param$(7) = "Leeward thrust"
Param$(8) = "Rotational thrust"
Param$(9) = "Heeling thrust"

YardName$(0) = "Fore"
YardName$(1) = "Main"
YardName$(2) = "Mizzen"
YardName$(3) = "Mizzen boom"

YardLevel$(1) = "Royal yards"
YardLevel$(2) = "Topgallant yards"
YardLevel$(3) = "Topsail yards"
YardLevel$(4) = "Lower yards"

StatusName$(1) = "Set"
StatusName$(2) = "1 reef"
StatusName$(3) = "2 reefs"
StatusName$(4) = "3 reefs"
StatusName$(5) = "Furled"
StatusName$(6) = "Blown out"

Sequence(1, 1) = 13 'draw first when wind from aft
Sequence(2, 1) = 1
Sequence(3, 1) = 2
Sequence(4, 1) = 3
Sequence(5, 1) = 4
Sequence(6, 1) = 5
Sequence(7, 1) = 6
Sequence(8, 1) = 7
Sequence(9, 1) = 8
Sequence(10, 1) = 9
Sequence(11, 1) = 10
Sequence(12, 1) = 11
Sequence(13, 1) = 12
Sequence(1, 2) = 9 'draw first when wind from fwd
Sequence(2, 2) = 10 'and from side with foreyard aft
Sequence(3, 2) = 11
Sequence(4, 2) = 12
Sequence(5, 2) = 5
Sequence(6, 2) = 6
Sequence(7, 2) = 7
Sequence(8, 2) = 8
Sequence(9, 2) = 1
Sequence(10, 2) = 2
Sequence(11, 2) = 3
Sequence(12, 2) = 4
Sequence(13, 2) = 13
Sequence(1, 3) = 9 'draw first when wind from fwd
Sequence(2, 3) = 10 'and from side with foreyard fwd
Sequence(3, 3) = 11
Sequence(4, 3) = 12
Sequence(5, 3) = 5
Sequence(6, 3) = 6
Sequence(7, 3) = 7
Sequence(8, 3) = 8
Sequence(9, 3) = 13
Sequence(10, 3) = 1
Sequence(11, 3) = 2
Sequence(12, 3) = 3
Sequence(13, 3) = 4

'initial values
WindSpeedIncr = 128
WindDirIncr = 128
HelmInstruction$ = "(await orders)"
YardInstruction$ = "(await orders)"
booEstablished = False
For i = 1 To 6: MessageTime(i) = 999999: Next i
MessageNr = 0
SailClicks = 0

'All the following parameters are replace from surprise.cfg (if present)

' Fraction of sail set by a particular SailStatus
SetFraction(1) = 1!
SetFraction(2) = 0.85
SetFraction(3) = 0.7
SetFraction(4) = 0.55
SetFraction(5) = 0!
SetFraction(6) = 0!

MaxYardAngle(0) = 60: 'Fore Yard
MaxYardAngle(1) = 60: 'Main Yard
MaxYardAngle(2) = 60: 'Crossjack
MaxYardAngle(3) = 45: 'Mizzen Boom

ShipLength = 146: 'feet
QDeckLength = 70
FocleLength = 34
GunDeckHeight = 15: 'height in feet above sealevel
QDeckHeight = 22 'height in feet above sealevel
ShipWidth = 38: 'feet
ShipDraught = 19: 'feet
ShipTons = 850: 'tons
MastDistance(1) = 52: 'Fore Mast
MastDistance(2) = -15: 'Main Mast
MastDistance(3) = -52: 'Mizzen Mast
MastDistance(4) = 134: 'Jib tack
LowestSailFoot(1) = 26 ' fore course
LowestSailFoot(2) = 25 ' main course
LowestSailFoot(3) = 26 ' mizzen sail
LowestSailFoot(4) = 35 ' jib
RudderArea = 64: 'square feet
MaxJibAngle = 30: 'Jib
MaxHelmAngle = 55: 'Helm

HelmAngle = 0: 'Helm: + = starboard helm = port rudder
Heading = 0: 'in degrees
Speed = 0: ' in knots
LeewaySpeed = 0
HeelingAngle = 0
MadeSail = 7: 'all sails but royals set
WindDirection = 180: ' in  (usual notation)
WindSpeed = 20: ' in knots
DisplayParam = 5: ' the currently displayed sail parameter
YardLevelShown = 4:  'yards to display in PlanView (4=courses; 3=topsail; 2=topg'l'nt; 1=royal)

MapWidth = 0.2: 'width of map in nautical miles
BowFactor = 0.3: 'modifies Frontal CSA (frontal aspect is not flat rectangle)
WaterDragFactor = 66: 'arbitrary scale factor for drag of water (viscosity of water = 66 * viscosity of air)
RudderFactor = 770: 'arbitrary scale factor for effectiveness of rudder vs sail (density of water = 770 * dens. air)
MassFactor = 1!: 'arbitrary conversion factor to convert tons to mass
RotatInertFactor = 1!: 'arbitrary scale factor for interia about vertical axis (turning)
RotatDragFactor = 200: 'arbitrary scale factor for rotational drag (turning)
HeelAngleFactor = 0.025: 'scale factor to generate appropriate heel
HeelInertFactor = 0.0002: 'arbitrary scale factor for inertia about longtitudinal axis (heel)
HeelDragFactor = 0.5: ' arbitrary scale factor for drag against heeling

'Square Sail Dimensions
HeadWidth(1) = 26
FootWidth(1) = 31
HeadWidth(2) = 31
FootWidth(2) = 42
HeadWidth(3) = 42
FootWidth(3) = 62
HeadWidth(4) = 60
FootWidth(4) = 60
HeadWidth(5) = 26
FootWidth(5) = 34
HeadWidth(6) = 34
FootWidth(6) = 47
HeadWidth(7) = 47
FootWidth(7) = 66
HeadWidth(8) = 66
FootWidth(8) = 66
HeadWidth(9) = 19
FootWidth(9) = 26
HeadWidth(10) = 26
FootWidth(10) = 36
HeadWidth(11) = 36
FootWidth(11) = 42
HeadWidth(12) = 50
FootWidth(12) = 50

SailDepth(1) = 9
SailDepth(2) = 21
SailDepth(3) = 35
SailDepth(4) = 32
SailDepth(5) = 10
SailDepth(6) = 26
SailDepth(7) = 40
SailDepth(8) = 35
SailDepth(9) = 8
SailDepth(10) = 20
SailDepth(11) = 43
SailDepth(12) = 32

'Jib dimensions: 1=large, 2=working 3=storm
JibDepth(1) = 60
JibFootWidth(1) = 45
JibTheorMaxSpeed(1) = 30
JibDepth(2) = 50
JibFootWidth(2) = 40
JibTheorMaxSpeed(2) = 40
JibDepth(3) = 40
JibFootWidth(3) = 20
JibTheorMaxSpeed(3) = 50
JibSet = 2

'Initial Sail Status: 1=set 5=furled
SailStatus(1) = 5
SailStatus(2) = 1
SailStatus(3) = 1
SailStatus(4) = 1
SailStatus(5) = 5
SailStatus(6) = 1
SailStatus(7) = 1
SailStatus(8) = 1
SailStatus(9) = 5
SailStatus(10) = 1
SailStatus(11) = 1
SailStatus(12) = 1
SailStatus(13) = 1

'Maximum number of reefs
MaxReefs(1) = 0
MaxReefs(2) = 0
MaxReefs(3) = 3
MaxReefs(4) = 2
MaxReefs(5) = 0
MaxReefs(6) = 0
MaxReefs(7) = 3
MaxReefs(8) = 2
MaxReefs(9) = 0
MaxReefs(10) = 0
MaxReefs(11) = 3
MaxReefs(12) = 2
MaxReefs(13) = 0

'Theoretical maximum wind speed for sail
TheorMaxSpeed(1) = 20
TheorMaxSpeed(2) = 25
TheorMaxSpeed(3) = 35
TheorMaxSpeed(4) = 50
TheorMaxSpeed(5) = 20
TheorMaxSpeed(6) = 25
TheorMaxSpeed(7) = 35
TheorMaxSpeed(8) = 50
TheorMaxSpeed(9) = 20
TheorMaxSpeed(10) = 25
TheorMaxSpeed(11) = 35
TheorMaxSpeed(12) = 50
TheorMaxSpeed(13) = 50

End Sub

Public Sub NewFrame(x1, y1, Wide, High, Style$, FrameWidth)
'Version date: 97.12.28

' "Style" should be "inward" or "outward"
' "FrameWidth" is the size you want the frame to be.

Dim st_l$
Dim LT_l, RB_l As Integer

st_l$ = LCase$(Left$(Style$, 1))

'Draw Highlight and Shadow lines
LT_l = 15: RB_l = 8
If st_l$ = "i" Then LT_l = 8: RB_l = 15
frmSurprise.Line (x1 - FrameWidth, y1 - FrameWidth)-(x1 + Wide + FrameWidth, y1 - FrameWidth), QBColor(LT_l)
frmSurprise.Line (x1 - FrameWidth, y1 - FrameWidth)-(x1 - FrameWidth, y1 + High + FrameWidth), QBColor(LT_l)
frmSurprise.Line (x1 + Wide + FrameWidth, y1 - FrameWidth)-(x1 + Wide + FrameWidth, y1 + High + FrameWidth), QBColor(RB_l)
frmSurprise.Line (x1 + Wide + FrameWidth, y1 + High + FrameWidth)-(x1 - FrameWidth, y1 + High + FrameWidth), QBColor(RB_l)

End Sub

Private Function Prnt(a$)
'Version date: 97.12.28
'Prints decimals with '.' in locales with ','
Dim Posn_l As Integer
Posn_l = InStr(a$, ",")
If Posn_l = 0 Then
  Prnt = a$
Else
  Prnt = Left(a$, Posn_l - 1) & "." & Mid$(a$, Posn_l + 1)
End If
End Function
